@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,15 +20,16 @@
 */
 #include "../SDL_internal.h"
 
-#ifndef _SDL_gesture_c_h
-#define _SDL_gesture_c_h
+#ifndef SDL_gesture_c_h_
+#define SDL_gesture_c_h_
 
 extern int SDL_GestureAddTouch(SDL_TouchID touchId);
+extern int SDL_GestureDelTouch(SDL_TouchID touchId);
 
 extern void SDL_GestureProcessEvent(SDL_Event* event);
 
-extern int SDL_RecordGesture(SDL_TouchID touchId);
+extern void SDL_GestureQuit(void);
 
-#endif /* _SDL_gesture_c_h */
+#endif /* SDL_gesture_c_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
