@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -110,5 +110,5 @@ static const unsigned char default_cmask[] = {
     0x03, 0x00
 };
 
-#endif /* TRUE_MACINTOSH_CURSOR */
+#endif /* USE_MACOS_CURSOR */
 /* vi: set ts=4 sw=4 expandtab: */
