@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,7 +19,12 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
+#ifndef SDL_haptic_c_h_
+#define SDL_haptic_c_h_
+
 extern int SDL_HapticInit(void);
 extern void SDL_HapticQuit(void);
 
+#endif /* SDL_haptic_c_h_ */
+
 /* vi: set ts=4 sw=4 expandtab: */
