@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,10 +20,10 @@
 */
 #include "../../SDL_internal.h"
 
-#if SDL_VIDEO_RENDER_OGL_ES2
+#ifndef SDL_shaders_gles2_h_
+#define SDL_shaders_gles2_h_
 
-#ifndef SDL_shaderdata_h_
-#define SDL_shaderdata_h_
+#if SDL_VIDEO_RENDER_OGL_ES2
 
 typedef struct GLES2_ShaderInstance
 {
@@ -46,15 +46,25 @@ typedef enum
     GLES2_SHADER_FRAGMENT_TEXTURE_ABGR_SRC,
     GLES2_SHADER_FRAGMENT_TEXTURE_ARGB_SRC,
     GLES2_SHADER_FRAGMENT_TEXTURE_BGR_SRC,
-    GLES2_SHADER_FRAGMENT_TEXTURE_RGB_SRC
+    GLES2_SHADER_FRAGMENT_TEXTURE_RGB_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_YUV_JPEG_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_YUV_BT601_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_YUV_BT709_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV12_JPEG_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV12_BT601_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV12_BT709_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV21_JPEG_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV21_BT601_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_NV21_BT709_SRC,
+    GLES2_SHADER_FRAGMENT_TEXTURE_EXTERNAL_OES_SRC
 } GLES2_ShaderType;
 
 #define GLES2_SOURCE_SHADER (GLenum)-1
 
-const GLES2_Shader *GLES2_GetShader(GLES2_ShaderType type, SDL_BlendMode blendMode);
-
-#endif /* SDL_shaderdata_h_ */
+const GLES2_Shader *GLES2_GetShader(GLES2_ShaderType type);
 
 #endif /* SDL_VIDEO_RENDER_OGL_ES2 */
 
+#endif /* SDL_shaders_gles2_h_ */
+
 /* vi: set ts=4 sw=4 expandtab: */
