@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -41,8 +41,6 @@
 static NativeWindowType hNativeWnd = 0; /* A handle to the window we will create. */
 #endif
 
-static SDL_bool PND_initialized = SDL_FALSE;
-
 static int
 PND_available(void)
 {
@@ -52,11 +50,11 @@ PND_available(void)
 static void
 PND_destroy(SDL_VideoDevice * device)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *) device->driverdata;
-
     if (device->driverdata != NULL) {
+        SDL_free(device->driverdata);
         device->driverdata = NULL;
     }
+    SDL_free(device);
 }
 
 static SDL_VideoDevice *
@@ -93,9 +91,8 @@ PND_create()
     phdata->egl_initialized = SDL_TRUE;
 
 
-    /* Setup amount of available displays and current display */
+    /* Setup amount of available displays */
     device->num_displays = 0;
-    device->current_display = 0;
 
     /* Set device free function */
     device->free = PND_destroy;
@@ -105,8 +102,8 @@ PND_create()
     device->VideoQuit = PND_videoquit;
     device->GetDisplayModes = PND_getdisplaymodes;
     device->SetDisplayMode = PND_setdisplaymode;
-    device->CreateWindow = PND_createwindow;
-    device->CreateWindowFrom = PND_createwindowfrom;
+    device->CreateSDLWindow = PND_createwindow;
+    device->CreateSDLWindowFrom = PND_createwindowfrom;
     device->SetWindowTitle = PND_setwindowtitle;
     device->SetWindowIcon = PND_setwindowicon;
     device->SetWindowPosition = PND_setwindowposition;
@@ -119,7 +116,9 @@ PND_create()
     device->RestoreWindow = PND_restorewindow;
     device->SetWindowGrab = PND_setwindowgrab;
     device->DestroyWindow = PND_destroywindow;
+#if 0
     device->GetWindowWMInfo = PND_getwindowwminfo;
+#endif
     device->GL_LoadLibrary = PND_gl_loadlibrary;
     device->GL_GetProcAddress = PND_gl_getprocaddres;
     device->GL_UnloadLibrary = PND_gl_unloadlibrary;
@@ -204,10 +203,6 @@ PND_createwindow(_THIS, SDL_Window * window)
 
     SDL_WindowData *wdata;
 
-    uint32_t winargc = 0;
-    int32_t status;
-
-
     /* Allocate window internal data */
     wdata = (SDL_WindowData *) SDL_calloc(1, sizeof(SDL_WindowData));
     if (wdata == NULL) {
@@ -292,7 +287,7 @@ PND_restorewindow(_THIS, SDL_Window * window)
 {
 }
 void
-PND_setwindowgrab(_THIS, SDL_Window * window)
+PND_setwindowgrab(_THIS, SDL_Window * window, SDL_bool grabbed)
 {
 }
 void
@@ -305,13 +300,14 @@ PND_destroywindow(_THIS, SDL_Window * window)
 /*****************************************************************************/
 /* SDL Window Manager function                                               */
 /*****************************************************************************/
+#if 0
 SDL_bool
 PND_getwindowwminfo(_THIS, SDL_Window * window, struct SDL_SysWMinfo *info)
 {
     if (info->version.major <= SDL_MAJOR_VERSION) {
         return SDL_TRUE;
     } else {
-        SDL_SetError("application not compiled with SDL %d.%d\n",
+        SDL_SetError("application not compiled with SDL %d.%d",
                      SDL_MAJOR_VERSION, SDL_MINOR_VERSION);
         return SDL_FALSE;
     }
@@ -319,6 +315,7 @@ PND_getwindowwminfo(_THIS, SDL_Window * window, struct SDL_SysWMinfo *info)
     /* Failed to get window manager information */
     return SDL_FALSE;
 }
+#endif
 
 /*****************************************************************************/
 /* SDL OpenGL/OpenGL ES functions                                            */
@@ -326,8 +323,6 @@ PND_getwindowwminfo(_THIS, SDL_Window * window, struct SDL_SysWMinfo *info)
 int
 PND_gl_loadlibrary(_THIS, const char *path)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
-
     /* Check if OpenGL ES library is specified for GF driver */
     if (path == NULL) {
         path = SDL_getenv("SDL_OPENGL_LIBRARY");
@@ -365,7 +360,6 @@ PND_gl_loadlibrary(_THIS, const char *path)
 void *
 PND_gl_getprocaddres(_THIS, const char *proc)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
     void *function_address;
 
     /* Try to get function address through the egl interface */
@@ -409,10 +403,7 @@ PND_gl_createcontext(_THIS, SDL_Window * window)
 {
     SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
     SDL_WindowData *wdata = (SDL_WindowData *) window->driverdata;
-    SDL_DisplayData *didata =
-        (SDL_DisplayData *) SDL_GetDisplayForWindow(window)->driverdata;
     EGLBoolean status;
-    int32_t gfstatus;
     EGLint configs;
     uint32_t attr_pos;
     EGLint attr_value;
@@ -629,12 +620,12 @@ PND_gl_createcontext(_THIS, SDL_Window * window)
     hNativeWnd = (NativeWindowType)malloc(16*1024);
 
     if(!hNativeWnd)
-        printf( "Error : Wiz framebuffer allocatation failed\n" );
+        printf( "Error: Wiz framebuffer allocatation failed\n" );
     else
-        printf( "SDL13: Wiz framebuffer allocated: %X\n", hNativeWnd );
+        printf( "SDL: Wiz framebuffer allocated: %X\n", hNativeWnd );
     }
     else {
-    printf( "SDL13: Wiz framebuffer already allocated: %X\n", hNativeWnd );
+        printf( "SDL: Wiz framebuffer already allocated: %X\n", hNativeWnd );
     }
 
     wdata->gles_surface =
@@ -650,7 +641,7 @@ PND_gl_createcontext(_THIS, SDL_Window * window)
 
 
     if (wdata->gles_surface == 0) {
-        SDL_SetError("Error : eglCreateWindowSurface failed;\n");
+        SDL_SetError("Error : eglCreateWindowSurface failed;");
         return NULL;
     }
 
@@ -787,18 +778,14 @@ PND_gl_getswapinterval(_THIS)
     return ((SDL_VideoData *) _this->driverdata)->swapinterval;
 }
 
-void
+int
 PND_gl_swapwindow(_THIS, SDL_Window * window)
 {
     SDL_VideoData *phdata = (SDL_VideoData *) _this->driverdata;
     SDL_WindowData *wdata = (SDL_WindowData *) window->driverdata;
-    SDL_DisplayData *didata =
-        (SDL_DisplayData *) SDL_GetDisplayForWindow(window)->driverdata;
-
 
     if (phdata->egl_initialized != SDL_TRUE) {
-        SDL_SetError("PND: GLES initialization failed, no OpenGL ES support");
-        return;
+        return SDL_SetError("PND: GLES initialization failed, no OpenGL ES support");
     }
 
     /* Many applications do not uses glFinish(), so we call it for them */
@@ -808,6 +795,7 @@ PND_gl_swapwindow(_THIS, SDL_Window * window)
     eglWaitGL();
 
     eglSwapBuffers(phdata->egl_display, wdata->gles_surface);
+    return 0;
 }
 
 void
@@ -838,7 +826,7 @@ PND_gl_deletecontext(_THIS, SDL_GLContext context)
     {
       free(hNativeWnd);
       hNativeWnd = 0;
-      printf( "SDL13: Wiz framebuffer released\n" );
+      printf( "SDL: Wiz framebuffer released\n" );
     }
 #endif
 
