@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,8 +19,8 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#ifndef _SDL_pspgl_c_h
-#define _SDL_pspgl_c_h
+#ifndef SDL_pspgl_c_h_
+#define SDL_pspgl_c_h_
 
 
 #include <GLES/egl.h>
@@ -40,7 +40,7 @@ extern void * PSP_GL_GetProcAddress(_THIS, const char *proc);
 extern int PSP_GL_MakeCurrent(_THIS,SDL_Window * window, SDL_GLContext context);
 extern void PSP_GL_SwapBuffers(_THIS);
 
-extern void PSP_GL_SwapWindow(_THIS, SDL_Window * window);
+extern int PSP_GL_SwapWindow(_THIS, SDL_Window * window);
 extern SDL_GLContext PSP_GL_CreateContext(_THIS, SDL_Window * window);
 
 extern int PSP_GL_LoadLibrary(_THIS, const char *path);
@@ -49,4 +49,6 @@ extern int PSP_GL_SetSwapInterval(_THIS, int interval);
 extern int PSP_GL_GetSwapInterval(_THIS);
 
 
-#endif /* _SDL_pspgl_c_h */
+#endif /* SDL_pspgl_c_h_ */
+
+/* vi: set ts=4 sw=4 expandtab: */
